==> sli-plots-0.0.29.tgz: package/addon/components/bar-graph.js <==
import Component from '@glimmer/component';
import { tracked } from '@glimmer/tracking';

export default class BarGraphComponent extends Component {
  @tracked lineElement;
  @tracked textHeight;

  get isLoading() {
    return this.args.isLoading;
  }

  get type() {
    return this.args.type || 'timeseries'; // timeseries or categorical
  }

  get isTimeseries() {
    return this.type === 'timeseries';
  }

  get color() {
    return this.args.color ? this.args.color : 'black'
  }

  get opacity() {
    return this.args.opacity ? this.args.opacity : '0.6'
  }

  get strokeWidth() {
    return this.args.strokeWidth ? this.args.strokeWidth : '0'
  }

  get strokeColor() {
    return this.args.strokeColor ? this.args.strokeColor : this.color
  }

  get xKey() {
    return this.args.xKey ? this.args.xKey : 'OCCURED';
  }

  get yKey() {
    return this.args.yKey ? this.args.yKey : 'SUM';
  }

  get showNoData() {
    return this.args.showNoData || false;
  }

  // Percent of graph height no data bars should be
  get noDataBarHeight() {
    return this.args.noDataBarHeight || 1;
  }

  get noDataFillColor() {
    return this.args.noDataFillColor || 'lightgray';
  }

  get aggregateScaleMultiplier() {
    const scale = this.args.aggregateScale || 'minute';
    switch (scale) {
      case 'week':
        return 24 * 60 * 60 * 1000 * 7;
      case 'day':
      case 'days':
        return 24 * 60 * 60 * 1000;
      case 'hour':
      case 'hours':
        return 60 * 60 * 1000;
      case 'minute':
      case 'minutes':
        return 60 * 1000;
      case 'second':
      case 'seconds':
        return 1000;
      case 'millisecond':
      case 'milliseconds':
      default:
        return 1
    }
  }

  // <title> elements inside of a rectangle show an accessibility tooltip like info box
  get hasBarTitle() {
    return !!this.args.hasBarTitle;
  }

  get barTitleTextKey() {
    return this.args.barTitleTextKey || 'at'
  }

  // percent of x width the bar is drawn as. If 100%, the bar will fill 100% of its width alloted (leavning no gap between bars)
  // if 50% the bar will be 50% of its max width and shifted to be centered
  get percentWidth() {
    return this.args.percentWidth || 70;
  }

  get barWidth() {
    return this.args.barWidth
      ? this.args.barWidth
      : this.autoWidth()
  }

  get hasBarText() { // bool
    return this.args.hasBarText || false;
  }

  get barTextColor() {
    return this.args.barTextColor || "white";
  }

  get barTextY() {
    return this.args.dimensions.height - (this.textHeight ? this.textHeight : 5);
  }

  get aggregatedTime() {
    return this.args.aggregatedTime || 60 * 60 * 1000;
  }

  get barAnchor() { // Anchor bar to start of timestamp, or center it ("center", "left")
    return this.args.barAnchor ? this.args.barAnchor : "left"
  }
  
  // Percent based padding for categorical bars - 10 = 10% padding 90% bar
  get categoryBarPadding() {
    return this.args.categoryBarPadding || 10;
  }

  get categoryBarPaddingWidth() {
    return (this.categoryBarPadding/100) * (this.args.dimensions.width / this.args.dataset.length);
  }
  
  get percentAdjustedBarWidth() {
    return this.barWidth * (this.percentWidth / 100);
  }

  autoWidth() {
    if(!this.args.dataset || !this.args.range || !this.args.dimensions.width) return '5';
    const isRaw = this.args.aggregatedTime === 0 && this.args.dataset.length > 1;
    if (isRaw) {
      return this.calculateRawWidth();
    } else if (this.isTimeseries) {
      return this.args.xScale(new Date(this.args.range[0].getTime() + this.aggregatedTime * this.aggregateScaleMultiplier)) - this.args.xScale(this.args.range[0]);
    } else {
      const percentWidth = this.args.dimensions.width / (this.args.dataset.length === 0 ? 1 : this.args.dataset.length);
      return percentWidth - this.categoryBarPaddingWidth;
    }
  }

  calculateRawWidth() {
    const datapointDate1 = new Date(this.args.dataset[0][this.xKey]);
    const datapointDate2 = new Date(this.args.dataset[1][this.xKey]);
    return Math.abs(this.args.xScale(datapointDate2) - this.args.xScale(datapointDate1));
  }

  calculateBarX(dataPoint, index) {
    if(this.isTimeseries) {
      const at = dataPoint[this.xKey]
      const timeLocation = this.args.xScale(typeof at === "string" ? new Date(at) : at);

      return this.barAnchor === "left"
        ? timeLocation
        : timeLocation - (this.barWidth / 2)
    } else {
      return ((this.args.dimensions.width / this.args.dataset.length) * index) + (this.categoryBarPaddingWidth / 2);
    }
  }

  setTextDimensions = (element) => {
    this.textHeight = element.clientHeight;
  }

  noDataBars() {
    const barCount = this.args.dimensions.width / this.barWidth;
    // const zero = this.args.dimensions.height - this.args.yScale(0);
    const barHeight = ((this.noDataBarHeight/100)) * this.args.dimensions.height;
    const barPaddingWidth = (this.barWidth - this.percentAdjustedBarWidth) / 2;
    const barY = this.args.dimensions.height - barHeight;

    return Array.from(Array(barCount)).map((_, i) => {
      const barX = (i * this.barWidth) + barPaddingWidth;
      return {
        index: i,
        value: null,
        time: null,
        barWidth: this.percentAdjustedBarWidth,
        barHeight,
        barY,
        barX,
        barTextX: barX,
        title: "",
        color: this.noDataFillColor
      };
    });
  }

  get bars() {
    if (this.args.dataset.length === 0 && this.showNoData) return this.noDataBars();
    const { xKey, yKey, barWidth, percentAdjustedBarWidth } = this;
    const zero = this.args.dimensions.height - this.args.yScale(0);

    return this.args.dataset.map((datapoint, index) => {
      if(datapoint[yKey] == null) return
      const valueHeight = this.args.yScale(parseFloat(datapoint[yKey]));
      const barX = this.calculateBarX(datapoint, index);
      const adjustedBarX = barX + (barWidth - percentAdjustedBarWidth) / 2;
      return {
        index,
        value: datapoint[yKey],
        time: this.isTimeseries ? datapoint[xKey] : null,
        barWidth: percentAdjustedBarWidth,
        barHeight: this.args.dimensions.height - (valueHeight + zero),
        barY: valueHeight,
        barX: adjustedBarX,
        barTextX: adjustedBarX,
        title: datapoint[this.barTitleTextKey],
        color: this.color
      }
    });
  }
}

==> sli-plots-0.0.31.tgz: package/addon/components/bar-graph.js <==
import Component from '@glimmer/component';
import { tracked } from '@glimmer/tracking';

export default class BarGraphComponent extends Component {
  @tracked lineElement;
  @tracked textHeight;

  get isLoading() {
    return this.args.isLoading;
  }

  get type() {
    return this.args.type || 'timeseries'; // timeseries or categorical
  }

  get isTimeseries() {
    return this.type === 'timeseries';
  }

  get color() {
    return this.args.color ? this.args.color : 'black'
  }

  get opacity() {
    return this.args.opacity ? this.args.opacity : '0.6'
  }

  get strokeWidth() {
    return this.args.strokeWidth ? this.args.strokeWidth : '0'
  }

  get strokeColor() {
    return this.args.strokeColor ? this.args.strokeColor : this.color
  }

  get xKey() {
    return this.args.xKey ? this.args.xKey : 'OCCURED';
  }

  get yKey() {
    return this.args.yKey ? this.args.yKey : 'SUM';
  }

  get showNoData() {
    return this.args.showNoData || false;
  }

  // Percent of graph height no data bars should be
  get noDataBarHeight() {
    return this.args.noDataBarHeight || 1;
  }

  get noDataFillColor() {
    return this.args.noDataFillColor || 'lightgray';
  }

  get aggregateScaleMultiplier() {
    const scale = this.args.aggregateScale || 'minute';
    switch (scale) {
      case 'week':
        return 24 * 60 * 60 * 1000 * 7;
      case 'day':
      case 'days':
        return 24 * 60 * 60 * 1000;
      case 'hour':
      case 'hours':
        return 60 * 60 * 1000;
      case 'minute':
      case 'minutes':
        return 60 * 1000;
      case 'second':
      case 'seconds':
        return 1000;
      case 'millisecond':
      case 'milliseconds':
      default:
        return 1
    }
  }

  // <title> elements inside of a rectangle show an accessibility tooltip like info box
  get hasBarTitle() {
    return !!this.args.hasBarTitle;
  }

  get barTitleTextKey() {
    return this.args.barTitleTextKey || 'at'
  }

  // percent of x width the bar is drawn as. If 100%, the bar will fill 100% of its width alloted (leavning no gap between bars)
  // if 50% the bar will be 50% of its max width and shifted to be centered
  get percentWidth() {
    return this.args.percentWidth || 70;
  }

  get barWidth() {
    return this.args.barWidth
      ? this.args.barWidth
      : this.autoWidth()
  }

  get hasBarText() { // bool
    return this.args.hasBarText || false;
  }

  get barTextColor() {
    return this.args.barTextColor || "white";
  }

  get barTextY() {
    return this.args.dimensions.height - (this.textHeight ? this.textHeight : 5);
  }

  get aggregatedTime() {
    return this.args.aggregatedTime || 60 * 60 * 1000;
  }

  get barAnchor() { // Anchor bar to start of timestamp, or center it ("center", "left")
    return this.args.barAnchor ? this.args.barAnchor : "left"
  }
  
  // Percent based padding for categorical bars - 10 = 10% padding 90% bar
  get categoryBarPadding() {
    return this.args.categoryBarPadding || 10;
  }

  get categoryBarPaddingWidth() {
    return (this.categoryBarPadding/100) * (this.args.dimensions.width / this.args.dataset.length);
  }
  
  get percentAdjustedBarWidth() {
    return this.barWidth * (this.percentWidth / 100);
  }

  autoWidth() {
    if(!this.args.dataset || !this.args.range || !this.args.dimensions.width) return '5';
    const isRaw = this.args.aggregatedTime === 0 && this.args.dataset.length > 1;
    if (isRaw) {
      return this.calculateRawWidth();
    } else if (this.isTimeseries) {
      return this.args.xScale(new Date(this.args.range[0].getTime() + this.aggregatedTime * this.aggregateScaleMultiplier)) - this.args.xScale(this.args.range[0]);
    } else {
      const percentWidth = this.args.dimensions.width / (this.args.dataset.length === 0 ? 1 : this.args.dataset.length);
      return percentWidth - this.categoryBarPaddingWidth;
    }
  }

  calculateRawWidth() {
    const datapointDate1 = new Date(this.args.dataset[0][this.xKey]);
    const datapointDate2 = new Date(this.args.dataset[1][this.xKey]);
    return Math.abs(this.args.xScale(datapointDate2) - this.args.xScale(datapointDate1));
  }

  calculateBarX(dataPoint, index) {
    if(this.isTimeseries) {
      const at = dataPoint[this.xKey]
      const timeLocation = this.args.xScale(typeof at === "string" ? new Date(at) : at);

      return this.barAnchor === "left"
        ? timeLocation
        : timeLocation - (this.barWidth / 2)
    } else {
      return ((this.args.dimensions.width / this.args.dataset.length) * index) + (this.categoryBarPaddingWidth / 2);
    }
  }

  setTextDimensions = (element) => {
    this.textHeight = element.clientHeight;
  }

  noDataBars() {
    const barCount = this.args.dimensions.width / this.barWidth;
    // const zero = this.args.dimensions.height - this.args.yScale(0);
    const barHeight = ((this.noDataBarHeight/100)) * this.args.dimensions.height;
    const barPaddingWidth = (this.barWidth - this.percentAdjustedBarWidth) / 2;
    const barY = this.args.dimensions.height - barHeight;

    return Array.from(Array(barCount)).map((_, i) => {
      const barX = (i * this.barWidth) + barPaddingWidth;
      return {
        index: i,
        value: null,
        time: null,
        barWidth: this.percentAdjustedBarWidth,
        barHeight,
        barY,
        barX,
        barTextX: barX,
        title: "",
        color: this.noDataFillColor
      };
    });
  }

  get bars() {
    if (this.args.dataset?.length === 0 && this.showNoData) return this.noDataBars();
    const { xKey, yKey, barWidth, percentAdjustedBarWidth } = this;
    const zero = this.args.dimensions.height - this.args.yScale(0);

    return this.args.dataset.map((datapoint, index) => {
      if(datapoint[yKey] == null) return
      const valueHeight = this.args.yScale(parseFloat(datapoint[yKey]));
      const barX = this.calculateBarX(datapoint, index);
      const adjustedBarX = barX + (barWidth - percentAdjustedBarWidth) / 2;
      return {
        index,
        value: datapoint[yKey],
        time: this.isTimeseries ? datapoint[xKey] : null,
        barWidth: percentAdjustedBarWidth,
        barHeight: this.args.dimensions.height - (valueHeight + zero),
        barY: valueHeight,
        barX: adjustedBarX,
        barTextX: adjustedBarX,
        title: datapoint[this.barTitleTextKey],
        color: this.color
      }
    });
  }
}
